@@ -1,4 +1,7 @@
 #include <fcntl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #include "anna.h"
 
 int get_lowmem_level (void) {
@@ -74,7 +77,7 @@ size_t package_to_choice(di_package *package, char *buf, size_t size) {
 	return n;
 }
 
-char *list_to_choices(di_package **packages) {
+char *list_to_choices(di_package **packages, int c_values) {
 	char buf[200], *ret;
 	int count = 0;
 	size_t ret_size = 1024, ret_used = 1, size;
@@ -83,7 +86,10 @@ char *list_to_choices(di_package **packages) {
 	ret = di_malloc(1024);
 	ret[0] = '\0';
 	while ((p = packages[count])) {
-		size = package_to_choice(p, buf, 200);
+		if (c_values)
+			size = choice_strcpy(buf, p->package, 200);
+		else
+			size = package_to_choice(p, buf, 200);
 		if (ret_used + size + 2 > ret_size) {
 			ret_size += 1024;
 			ret = di_realloc(ret, ret_size);
@@ -120,6 +126,75 @@ int configure_package (const char *package) {
 	return ret;
 }
 
+#ifdef LOADTEMPLATES
+int load_templates (di_packages *packages) {
+	di_slist_node *node;
+	size_t command_size = 1024, command_len;
+	char *command = di_malloc(command_size);
+	bool found_templates = false;
+	int ret;
+
+	if (!command)
+		return 0;
+
+	strcpy(command, "debconf-loadtemplate d-i");
+	command_len = strlen(command);
+
+	for (node = packages->list.head; node; node = node->next) {
+		di_package *package = node->data;
+		char *arg = NULL;
+		struct stat st;
+
+		if (package->type != di_package_type_real_package ||
+		    package->status_want != di_package_status_want_install)
+			continue;
+		if (asprintf(&arg, "%s/%s.templates",
+			     INFO_DIR, package->package) == -1) {
+			di_free(command);
+			return 0;
+		}
+		if (stat(arg, &st) == -1) {
+			free(arg);
+			continue;
+		}
+		while (command_len + 1 + strlen(arg) >= command_size) {
+			command_size *= 2;
+			command = di_realloc(command, command_size);
+			if (!command) {
+				free(arg);
+				return 0;
+			}
+		}
+		command[command_len] = ' ';
+		strcpy(command + command_len + 1, arg);
+		command_len += 1 + strlen(arg);
+		free(arg);
+		found_templates = true;
+	}
+	if (!found_templates)
+		return 0;
+	ret = !di_exec_shell_log(command);
+	di_free(command);
+
+	/* Delete templates after loading. */
+	for (node = packages->list.head; node; node = node->next) {
+		di_package *package = node->data;
+		char *arg = NULL;
+
+		if (package->type != di_package_type_real_package ||
+		    package->status_want != di_package_status_want_install)
+			continue;
+		if (asprintf(&arg, "%s/%s.templates",
+			     INFO_DIR, package->package) == -1)
+			return 0;
+		unlink(arg);
+		free(arg);
+	}
+
+	return ret;
+}
+#endif /* LOADTEMPLATES */
+
 /* Check whether the md5sum of file matches sum. If not, return 0. */
 int md5sum(const char *sum, const char *file) {
 	FILE *fp;
